package dmlib;
require Exporter;

@ISA    = qw(Exporter);
@EXPORT = qw(chk_armstatus);
@EXPORT = qw(chk_rearmstatus);
@EXPORT = qw(send_alert);
@EXPORT = qw(parse_gen_msg);
@EXPORT = qw(write_action);
@EXPORT = qw(write_tolog);
@EXPORT = qw(gethost);
@EXPORT = qw(run_unix_command);
@EXPORT = qw(exec_unix_command);


######################################################################
## Library:   tivoli_lib.pm                                          #
## Purpose:   to provide standard functions to RB monitoring scripts #
######################################################################

######################################################################
# Sub:       chk_armstatus                                           #
# Purpose:   check arm status                                        #
# Usage:     chk_armstatus($armfile, $armvalue)                      #
#            1) armfile (full path name)                             #
#            2) number of times to send alarm (eg. 5)                #
# Returns:   sendarm (1 if you need to send alert)                   # 
######################################################################

sub chk_armstatus {
    my $class = shift @_;
    my ($armfile, $armvalue) = @_;        
    my $armcount;

    my $sendarm = 0;
       #  process armfile 
       if (!(-e $armfile)) {
            open (ARMFILE, ">$armfile") || die ("Cannot create armfile $armfile\n");
            print ARMFILE ('1'); 
            close ARMFILE;
            $sendarm = 1;
       }
       else {
           (open(ARMFILE, "$armfile")) || die("Cannot open armfile $armfile\n");
           $armcount = <ARMFILE>;
           close ARMFILE;

           if ($armcount < $armvalue) {
               $armcount = $armcount + 1;
               open(ARMFILE, ">$armfile");
               print ARMFILE ($armcount);
               close ARMFILE;
               $sendarm = 1;
           }
    }
    return ($sendarm);
}

######################################################################
# Sub:       chk_rearmstatus                                         #
# Purpose:   check rearm status                                      #
# Usage:     chk_rearmstatus($armfile)                               #
#            1) armfile (full path name)                             #
# Returns:   sendarm (1 if you need to send alertto close)           #
######################################################################

sub chk_rearmstatus {
       my $class = shift @_;
       my ($armfile) = @_;        
       if (-e $armfile) {
               unlink ($armfile);
               return (1);
        }
       return (0);
}

######################################################################
# Sub:       send_alert                                              #
# Purpose:   send alert using appropriate method                     #
# Usage:     send_alert(11 parameters listed below)                  #
#            1) message                                              #
#            2) probe argument                                       #
#            3) method (postemsg, email...)                          #                        
#            4) severity                                             #
#            5) class                                                #
#            6) source                                               #
#            7) hostname                                             #
#            8) path_src                                             #
#            9) eventlog                                             # 
#           10) lockfile                                             # 
#           11) tecserver                                            #
# Returns:   sendarm (1 if you need to send alert)                   #
######################################################################

sub send_alert {
    my $classe = shift @_;
    my ($message, $pname, $method, $severity, $class, $source, $host, $path_src, $eventlog, $lockfile, $tecserver) = @_;
    my (%sev, $netview);

use Time::localtime;

            # Convert Tivoli severity to NetView6000 severity
            $sev{'CRITICAL'} = 'C';
            $sev{'FATAL'} = 'C';
            $sev{'WARNING'} = 'W';
            $sev{'UNKNOWN'} = 'W';
            $sev{'HARMLESS'} = 'X';

            # In procduction the netview hostname will be uanvp00
            $netview = 'netview6000';

            my $rdate=ctime();

            # Replace "blank" (if exists) by "_" otherwise, postemsg
            # fails
            #
            ($pname =~ / /) && ($pname =~ s/ /_/g);

            ($source =~ /VVPOST/) && ($host =~ /uagplusp10/) && ($source = "GPPOST");
            ($source =~ /VVPOST/) && ($host =~ /uagplusp30/) && ($source = "GPPOST");
            ($source =~ /VVPOST/) && ($host =~ /uagplusp40/) && ($source = "GPPOST");
            ($source =~ /VVPOST/) && ($host =~ /uhbprst/) && ($source = "RBBPRS");

           if ($method eq 'postemsg') {
             `"$path_src"postemsg -S $tecserver -m "$message" -r $severity probe_arg="$pname" hostname="$host" $class $source`;
           }
           elsif ($method eq 'wpostemsg') { 
           `"$path_src"wpostemsg -S $tecserver -m "$message" -r $severity probe_arg="$pname" hostname="$host" $class $source`;
           }
           elsif ($method eq 'email') {
           `nohup "$path_src"common/tecemail.pl $rdate $source $severity $host "$message" &`; 
           }
           elsif ($method eq 'logfile') { 
           }
           elsif ($method eq 'snmptrap') { 
                 if ($sev{$severity} eq 'HARMLESS') {
                   $message = "C".$sev{$severity}."XX"."$message";
           `"$path_src"snmptrap $netview 0 1.3.6.1.4.1.2.590.1 $host 6 603 0 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII "message" 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII "-" 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII $pname`;
                 }
                 else {
                   $message = "C".$sev{$severity}."EM"."$message";
           `"$path_src"snmptrap $netview 0 1.3.6.1.4.1.2.590.1 $host 6 602 0 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII "message" 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII "-" 1.3.6.1.4.1.2.6.12.5.1.1.17 OctetStringASCII $pname`;

                 }
           }
           else
	   {
             print("Unknown method: $method\n");
           }

            $message = $rdate."  ".$message;
            &write_to_log($message, $eventlog, $lockfile);

            # Pass the action_id to Sentry
            return (1);
}

######################################################################
# Sub:      parse_gen_msg                                            #
# Purpose:  Parse the message replace $ with the approprite values   #
# Usage:    parse_gen_msge (message text,                            #
#                   first replacement variable,                      #
#                   second replacment variable)                      #
######################################################################

sub parse_gen_msg {
      my $class = shift @_;
      my($msg, $var1, $var2) = @_;

      # Replace the first $ sign by the first parm
      # Replace the second $ sign by the second parm
      # Get rid of blank space and the carriage return
      # Get rid of trailing and preceeding blanks

      $msg =~ s/\$/$var1/;
      ($msg =~ /\$/) && ($msg =~ s/\$/$var2/);
      $msg =~ tr/\n//d ; 
      $msg =~ s/  / /g;
      $msg =~ s/  / /g;
      $msg =~ s/^ //g;
      $msg =~ s/ $//g;

      return ($msg);
}

########################################################################
# Sub:       write_action                                              #
# Purpose:   store scripts in a file for unix_exec.pl to run           #
# Usage:     write_action (script or command to run, logfile, lockfile)#
########################################################################

sub write_action {
    my $class = shift @_;
    my($msg, $actionlog, $lockfile) = @_;

    my $c=0;
    while (-e $lockfile && $c<5) {
           sleep 2;
           $c++;
    }
    if ($c>4) { die ("Cannot open lockfile $lockfile\n");    }

  # lock an access to the event logfile
    open(LOCK, ">$lockfile") || die ("Cannot open lockfile $lockfile\n");
    close (LOCK);
 
  # write error message to the event logfile
    open(DATA, ">>$actionlog") || die ("Cannot open actionlog $actionlog\n");
    print DATA "$msg\n";
    close (DATA);

  # unlock an access to the event logfile
    unlink($lockfile);
}

########################################################################
# Sub:        write_to_log                                             #
# Purpose:    Write message to the event log for trace purpose         #
# Usage:      write_to_log (message, eventlog, lockfile)               #
#             No exported function, called by other functions within   #
#             the library.                                             #
# Returns:    none                                                     #
########################################################################

sub write_to_log {
    my($msg, $eventlog, $lockfile) = @_;
    my $c=0;

    while (-e $lockfile && $c<5) {
           sleep 2;
	   $c++;
    }
    if ($c>4) { die ("Cannot1 open lockfile $lockfile\n");    }

  # lock an access to the event logfile
    open(LOCK, ">$lockfile") || die ("Cannot2 open lockfile $lockfile\n");
    close (LOCK);
 
  # write error message to the event logfile
    open(DATA, ">>$eventlog") || die ("Cannot3 open eventlog $eventlog\n");
    print DATA "$msg\n";
    close (DATA);

  # unlock an access to the event logfile
    unlink($lockfile);
}

########################################################################
# Sub:        write_tolog                                              #
# Purpose:    Write message to the event log for trace purpose         #
# Usage:      write_to_log (message, eventlog, lockfile)               #
#             Exported library.                                        #
# Returns:    none                                                     #
########################################################################

sub write_tolog {
    my $class = shift @_;
    my($msg, $eventlog, $lockfile) = @_;
    my $c=0;

    while (-e $lockfile && $c<5) {
           sleep 2;
	   $c++;
    }
    if ($c>4) { die ("Cannot1 open lockfile $lockfile\n");    }

  # lock an access to the event logfile
    open(LOCK, ">$lockfile") || die ("Cannot2 open lockfile $lockfile\n");
    close (LOCK);
 
  # write error message to the event logfile
    open(DATA, ">>$eventlog") || die ("Cannot3 open eventlog $eventlog\n");
    print DATA "$msg\n";
    close (DATA);

  # unlock an access to the event logfile
    unlink($lockfile);
}

########################################################################
# Sub:     gethost                                                     #
# Params:  none                                                        #
# Returns: local hostname                                              #
# Purpose: get local hostname and check wlocalhosts                    #
########################################################################

sub gethost {
    my $class = shift @_;

use Sys::Hostname;
my $wlocalhostfile = "/etc/wlocalhost";

my $host=hostname;

#  Check wlocalhost file
if (-e $wlocalhostfile) {
   open(HOSTFILE, "$wlocalhostfile") || die("Cannot open wlocalhost file $wlocalhostfile\n");
   $host = <HOSTFILE>;
   close HOSTFILE;
   $host =~ tr/\n//d ;

}

return $host;
}

########################################################################
# Sub:      exec_unix_command                                          #
# Usage:    exec_unix_command ($command)                               #
# Purpose:  execute the unix command in background                     #
#           log unix command in the event_log file                     #
########################################################################
       
sub exec_unix_command {
       my $class = shift @_;
       my ($eventlog, $lockfile, $command) = @_;
       my (@cmd);

use Time::localtime;

$command =~ s/^[ \t]+//;
@cmd = split(/[\t \n]+/, $command);

  if ( -x $cmd[0] ) {
        system("nohup $command & > /dev/null 2>&1");
        $message = "execute: $command";
  }
  else {
        $message = "cannot execute: $command"; 
  }

  $message = ctime()."  ".$message;
  &write_to_log($message, $eventlog, $lockfile);

return 0;
}

########################################################################
# Sub:      run_unix_command                                           #
# Usage:    run_unix_command ($command)                                #
# Returns:  1 if failed                                                #
# Purpose:  give the unix command 10 seconds to run or return fail     #
########################################################################
       
sub run_unix_command {
       my $class = shift @_;
       my ($command) = @_;

#  this function gives the command 10 seconds to run or it fails
$SIG{ALRM} = \&timed_out;
eval {
  alarm(45);
  system($command);
  alarm(0);
};
if ($@ =~ /got tired of waiting/) {
   return 1;
}
return 0;
}

sub timed_out {
  die "got tired of waiting";
}


########################################################################
#  Allows creation of the object of class tivoli_lib                   #
########################################################################

sub new {
        my $this = {};
        bless $this;
        return $this;
}

1;
